# -*- coding: utf-8 -*-
from setuphelpers import *
import time


r"""
Sources:
https://docs.microsoft.com/windows-hardware/manufacture/desktop/windows-setup-command-line-options
https://www.prajwaldesai.com/windows-10-upgrade-setup-failed-with-exit-code-0xc1900200/
https://www.tomshardware.com/how-to/bypass-windows-11-tpm-requirement
https://docs.microsoft.com/windows-hardware/manufacture/desktop/windows-setup-command-line-options?view=windows-11#compat
https://docs.microsoft.com/en-us/windows-server/administration/windows-commands/cleanmgr

Infos:
Forced install will automatically reboot the PC
This package can be updated graphically

"""
AllowUpgradesWithUnsupportedTPMOrCPU = False
DynamicUpdate = False
upgrade_path = makepath(systemdrive, "WindowsUpgrade")
upgrade_logs_path = makepath(systemdrive, "WindowsUpgradeLogs")
windows_old_path = makepath(systemdrive, "Windows.old")
cleanmgr_timeout = 660


def install():
    # Declaring local variables
    package_version = control.get_software_version()
    iso_path = makepath(basedir, "iso")
    cleanmgr_keys = [
        "Active Setup Temp Folders",
        "BranchCache",
        "Content Indexer Cleaner",
        "D3D Shader Cache",
        "Delivery Optimization Files",
        "Device Driver Packages",
        "Diagnostic Data Viewer database files",
        "Downloaded Program Files",
        "DownloadsFolder",
        "Internet Cache Files",
        "Language Pack",
        "Offline Pages Files",
        "Old ChkDsk Files",
        "Previous Installations",
        "Recycle Bin",
        "RetailDemo Offline Content",
        "Setup Log Files",
        "System error memory dump files",
        "System error minidump files",
        "Temporary Files",
        "Temporary Setup Files",
        "Temporary Sync Files",
        "Thumbnail Cache",
        "Update Cleanup",
        "Upgrade Discarded Files",
        "User file versions",
        "Windows Defender",
        "Windows Error Reporting Files",
        "Windows ESD installation files",
        "Windows Upgrade Log Files",
    ]

    # Checking if Windows must be upgraded
    if Version(windows_version(), 3) < Version(package_version, 3):
        # Trying to remove previous version of Windows if needed
        if isdir(windows_old_path) or isdir(r"C:\$Windows.~WS"):
            key_path = r"SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\VolumeCaches"
            key_value = r"StateFlags0042"
            for key in cleanmgr_keys:
                if "Upgrade Discarded Files" in key or "Previous Installations" in key:
                    registry_set(HKEY_LOCAL_MACHINE, makepath(key_path, key), key_value, 2)
                else:
                    registry_set(HKEY_LOCAL_MACHINE, makepath(key_path, key), key_value, 0)

            print("Removing previous version of Windows")
            cleanmgr_cmd = "cleanmgr /verylowdisk /sagerun:42"
            run_notfatal(cleanmgr_cmd, timeout=cleanmgr_timeout)  # seems to timeout on missing user graphical interface
            # run_powershell(r"Start-Process -FilePath CleanMgr.exe -ArgumentList '/sagerun:42 /verylowdisk' -WindowStyle Hidden -Wait")

            print("Waiting for custom CleanMgr")
            max_loop = cleanmgr_timeout
            while isrunning("cleanmgr"):
                loop = 0
                loop += 10
                if loop > max_loop:
                    print("CleanMgr timed out after %s seconds, trying to continue" % max_loop)
                    print('If the upgrade fail, please run: "%s" manually as asministrator on the PC' % cleanmgr_cmd)
                time.sleep(10)
                print("CleanMgr is running since %s seconds" % loop)
            print("CleanMgr finished")

        # Move iso
        if isdir(upgrade_path):
            remove_tree(upgrade_path)
        os.rename(iso_path, upgrade_path)

        if not isdir(upgrade_logs_path):
            mkdirs(upgrade_logs_path)

        # Upgrading Windows if needed
        if isdir(upgrade_path):
            print("%s folder exists, continuing..." % upgrade_path)
            print("Free disk space: %d bytes" % get_disk_free_space(upgrade_path))
            if get_disk_free_space(upgrade_path) < 25000000000:
                error("Not enough free space on disk for upgrade, please cleanup")
            else:
                # Changing cmd options
                additionnal_cmd = ""
                if AllowUpgradesWithUnsupportedTPMOrCPU:
                    registry_set(HKEY_LOCAL_MACHINE, r"SYSTEM\Setup\MoSetup", "AllowUpgradesWithUnsupportedTPMOrCPU", 1)
                    additionnal_cmd = additionnal_cmd + " /compat ignorewarning"
                if DynamicUpdate:
                    additionnal_cmd = additionnal_cmd + " /dynamicupdate enable"
                else:
                    additionnal_cmd = additionnal_cmd + " /dynamicupdate disable"

                if force:
                    try:
                        print("Force Upgrading Windows from version: %s to Windows version: %s" % (windows_version(), package_version))
                        run(
                            rf'"{makepath(upgrade_path, "setup.exe")}" /auto upgrade /quiet /eula accept /telemetry disable /showoobe none /copylogs "{makepath(upgrade_logs_path, "win11_upgrade_logs")}" {additionnal_cmd}',
                            timeout=5000,
                        )
                    except Exception as error_force_upgrade:
                        error(error_force_upgrade)
                else:
                    additionnal_cmd = additionnal_cmd + " /noreboot"
                    try:
                        print("Upgrading Windows from version: %s to Windows version: %s" % (windows_version(), package_version))
                        run(
                            rf'"{makepath(upgrade_path, "setup.exe")}" /auto upgrade /quiet /eula accept /telemetry disable /showoobe none /copylogs "{makepath(upgrade_logs_path, "win11_upgrade_logs")}" {additionnal_cmd}',
                            timeout=5000,
                        )
                    except Exception as error_upgrade:
                        error(error_upgrade)
        else:
            error("%s do not exist, please reinstall this package" % upgrade_path)
    else:
        print("This PC is already up-to-date on Windows version: %s" % windows_version())


def uninstall():
    if AllowUpgradesWithUnsupportedTPMOrCPU:
        registry_delete(HKEY_LOCAL_MACHINE, r"SYSTEM\Setup\MoSetup", "AllowUpgradesWithUnsupportedTPMOrCPU")
    if isdir(upgrade_path):
        print("Removing: %s" % upgrade_path)
        remove_tree(upgrade_path)
    if isdir(upgrade_logs_path):
        print("Removing: %s" % upgrade_logs_path)
        remove_tree(upgrade_logs_path)


def audit():
    if Version(windows_version()) < Version(control.get_software_version()):
        if isrunning("SetupHost"):
            print("WARNING: Windows upgrade is running...")
            return "WARNING"
        elif is_pending_reboot():
            print("WARNING: Please reboot this PC to finalize the upgrade")
            return "WARNING"
        else:
            print("ERROR: Windows upgrade failed")
            return "ERROR"
    else:
        print("OK: This PC is now up-to-date on Windows version: %s" % windows_version())
        print("Cleaning up since the upgrade was successful...")
        if isdir(upgrade_path):
            print("Removing: %s" % upgrade_path)
            remove_tree(upgrade_path)
        if isdir(upgrade_logs_path):
            print("Removing: %s" % upgrade_logs_path)
            remove_tree(upgrade_logs_path)
        return "OK"
